# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from braintrust_api import Braintrust, AsyncBraintrust
from braintrust_api.types import (
    Function,
)
from braintrust_api.pagination import SyncListObjects, AsyncListObjects

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFunctions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Braintrust) -> None:
        function = client.functions.create(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Braintrust) -> None:
        function = client.functions.create(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
            description="description",
            prompt_data={
                "prompt": {
                    "type": "completion",
                    "content": "content",
                },
                "options": {
                    "model": "model",
                    "params": {
                        "use_cache": True,
                        "temperature": 0,
                        "top_p": 0,
                        "max_tokens": 0,
                        "frequency_penalty": 0,
                        "presence_penalty": 0,
                        "response_format": {"type": "json_object"},
                        "tool_choice": "auto",
                        "function_call": "auto",
                        "n": 0,
                        "stop": ["string", "string", "string"],
                    },
                    "position": "position",
                },
                "origin": {
                    "prompt_id": "prompt_id",
                    "project_id": "project_id",
                    "prompt_version": "prompt_version",
                },
            },
            tags=["string", "string", "string"],
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Braintrust) -> None:
        response = client.functions.with_raw_response.create(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Braintrust) -> None:
        with client.functions.with_streaming_response.create(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Braintrust) -> None:
        function = client.functions.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Braintrust) -> None:
        response = client.functions.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Braintrust) -> None:
        with client.functions.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Braintrust) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `function_id` but received ''"):
            client.functions.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Braintrust) -> None:
        function = client.functions.update(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Braintrust) -> None:
        function = client.functions.update(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            function_data={"type": "prompt"},
            name="name",
            prompt_data={
                "prompt": {
                    "type": "completion",
                    "content": "content",
                },
                "options": {
                    "model": "model",
                    "params": {
                        "use_cache": True,
                        "temperature": 0,
                        "top_p": 0,
                        "max_tokens": 0,
                        "frequency_penalty": 0,
                        "presence_penalty": 0,
                        "response_format": {"type": "json_object"},
                        "tool_choice": "auto",
                        "function_call": "auto",
                        "n": 0,
                        "stop": ["string", "string", "string"],
                    },
                    "position": "position",
                },
                "origin": {
                    "prompt_id": "prompt_id",
                    "project_id": "project_id",
                    "prompt_version": "prompt_version",
                },
            },
            tags=["string", "string", "string"],
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Braintrust) -> None:
        response = client.functions.with_raw_response.update(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Braintrust) -> None:
        with client.functions.with_streaming_response.update(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Braintrust) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `function_id` but received ''"):
            client.functions.with_raw_response.update(
                function_id="",
            )

    @parametrize
    def test_method_list(self, client: Braintrust) -> None:
        function = client.functions.list()
        assert_matches_type(SyncListObjects[Function], function, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Braintrust) -> None:
        function = client.functions.list(
            ending_before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            function_name="function_name",
            ids="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            limit=0,
            org_name="org_name",
            project_name="project_name",
            slug="slug",
            starting_after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            version="version",
        )
        assert_matches_type(SyncListObjects[Function], function, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Braintrust) -> None:
        response = client.functions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = response.parse()
        assert_matches_type(SyncListObjects[Function], function, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Braintrust) -> None:
        with client.functions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = response.parse()
            assert_matches_type(SyncListObjects[Function], function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Braintrust) -> None:
        function = client.functions.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Braintrust) -> None:
        response = client.functions.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Braintrust) -> None:
        with client.functions.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Braintrust) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `function_id` but received ''"):
            client.functions.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_feedback(self, client: Braintrust) -> None:
        function = client.functions.feedback(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            feedback=[{"id": "id"}, {"id": "id"}, {"id": "id"}],
        )
        assert function is None

    @parametrize
    def test_raw_response_feedback(self, client: Braintrust) -> None:
        response = client.functions.with_raw_response.feedback(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            feedback=[{"id": "id"}, {"id": "id"}, {"id": "id"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = response.parse()
        assert function is None

    @parametrize
    def test_streaming_response_feedback(self, client: Braintrust) -> None:
        with client.functions.with_streaming_response.feedback(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            feedback=[{"id": "id"}, {"id": "id"}, {"id": "id"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = response.parse()
            assert function is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_feedback(self, client: Braintrust) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `function_id` but received ''"):
            client.functions.with_raw_response.feedback(
                function_id="",
                feedback=[{"id": "id"}, {"id": "id"}, {"id": "id"}],
            )

    @parametrize
    def test_method_replace(self, client: Braintrust) -> None:
        function = client.functions.replace(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_method_replace_with_all_params(self, client: Braintrust) -> None:
        function = client.functions.replace(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
            description="description",
            prompt_data={
                "prompt": {
                    "type": "completion",
                    "content": "content",
                },
                "options": {
                    "model": "model",
                    "params": {
                        "use_cache": True,
                        "temperature": 0,
                        "top_p": 0,
                        "max_tokens": 0,
                        "frequency_penalty": 0,
                        "presence_penalty": 0,
                        "response_format": {"type": "json_object"},
                        "tool_choice": "auto",
                        "function_call": "auto",
                        "n": 0,
                        "stop": ["string", "string", "string"],
                    },
                    "position": "position",
                },
                "origin": {
                    "prompt_id": "prompt_id",
                    "project_id": "project_id",
                    "prompt_version": "prompt_version",
                },
            },
            tags=["string", "string", "string"],
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_raw_response_replace(self, client: Braintrust) -> None:
        response = client.functions.with_raw_response.replace(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    def test_streaming_response_replace(self, client: Braintrust) -> None:
        with client.functions.with_streaming_response.replace(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncFunctions:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.create(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.create(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
            description="description",
            prompt_data={
                "prompt": {
                    "type": "completion",
                    "content": "content",
                },
                "options": {
                    "model": "model",
                    "params": {
                        "use_cache": True,
                        "temperature": 0,
                        "top_p": 0,
                        "max_tokens": 0,
                        "frequency_penalty": 0,
                        "presence_penalty": 0,
                        "response_format": {"type": "json_object"},
                        "tool_choice": "auto",
                        "function_call": "auto",
                        "n": 0,
                        "stop": ["string", "string", "string"],
                    },
                    "position": "position",
                },
                "origin": {
                    "prompt_id": "prompt_id",
                    "project_id": "project_id",
                    "prompt_version": "prompt_version",
                },
            },
            tags=["string", "string", "string"],
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncBraintrust) -> None:
        response = await async_client.functions.with_raw_response.create(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = await response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncBraintrust) -> None:
        async with async_client.functions.with_streaming_response.create(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = await response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncBraintrust) -> None:
        response = await async_client.functions.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = await response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncBraintrust) -> None:
        async with async_client.functions.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = await response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncBraintrust) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `function_id` but received ''"):
            await async_client.functions.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.update(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.update(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            function_data={"type": "prompt"},
            name="name",
            prompt_data={
                "prompt": {
                    "type": "completion",
                    "content": "content",
                },
                "options": {
                    "model": "model",
                    "params": {
                        "use_cache": True,
                        "temperature": 0,
                        "top_p": 0,
                        "max_tokens": 0,
                        "frequency_penalty": 0,
                        "presence_penalty": 0,
                        "response_format": {"type": "json_object"},
                        "tool_choice": "auto",
                        "function_call": "auto",
                        "n": 0,
                        "stop": ["string", "string", "string"],
                    },
                    "position": "position",
                },
                "origin": {
                    "prompt_id": "prompt_id",
                    "project_id": "project_id",
                    "prompt_version": "prompt_version",
                },
            },
            tags=["string", "string", "string"],
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncBraintrust) -> None:
        response = await async_client.functions.with_raw_response.update(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = await response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncBraintrust) -> None:
        async with async_client.functions.with_streaming_response.update(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = await response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncBraintrust) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `function_id` but received ''"):
            await async_client.functions.with_raw_response.update(
                function_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.list()
        assert_matches_type(AsyncListObjects[Function], function, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.list(
            ending_before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            function_name="function_name",
            ids="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            limit=0,
            org_name="org_name",
            project_name="project_name",
            slug="slug",
            starting_after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            version="version",
        )
        assert_matches_type(AsyncListObjects[Function], function, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncBraintrust) -> None:
        response = await async_client.functions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = await response.parse()
        assert_matches_type(AsyncListObjects[Function], function, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncBraintrust) -> None:
        async with async_client.functions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = await response.parse()
            assert_matches_type(AsyncListObjects[Function], function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncBraintrust) -> None:
        response = await async_client.functions.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = await response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncBraintrust) -> None:
        async with async_client.functions.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = await response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncBraintrust) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `function_id` but received ''"):
            await async_client.functions.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_feedback(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.feedback(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            feedback=[{"id": "id"}, {"id": "id"}, {"id": "id"}],
        )
        assert function is None

    @parametrize
    async def test_raw_response_feedback(self, async_client: AsyncBraintrust) -> None:
        response = await async_client.functions.with_raw_response.feedback(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            feedback=[{"id": "id"}, {"id": "id"}, {"id": "id"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = await response.parse()
        assert function is None

    @parametrize
    async def test_streaming_response_feedback(self, async_client: AsyncBraintrust) -> None:
        async with async_client.functions.with_streaming_response.feedback(
            function_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            feedback=[{"id": "id"}, {"id": "id"}, {"id": "id"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = await response.parse()
            assert function is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_feedback(self, async_client: AsyncBraintrust) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `function_id` but received ''"):
            await async_client.functions.with_raw_response.feedback(
                function_id="",
                feedback=[{"id": "id"}, {"id": "id"}, {"id": "id"}],
            )

    @parametrize
    async def test_method_replace(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.replace(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_method_replace_with_all_params(self, async_client: AsyncBraintrust) -> None:
        function = await async_client.functions.replace(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
            description="description",
            prompt_data={
                "prompt": {
                    "type": "completion",
                    "content": "content",
                },
                "options": {
                    "model": "model",
                    "params": {
                        "use_cache": True,
                        "temperature": 0,
                        "top_p": 0,
                        "max_tokens": 0,
                        "frequency_penalty": 0,
                        "presence_penalty": 0,
                        "response_format": {"type": "json_object"},
                        "tool_choice": "auto",
                        "function_call": "auto",
                        "n": 0,
                        "stop": ["string", "string", "string"],
                    },
                    "position": "position",
                },
                "origin": {
                    "prompt_id": "prompt_id",
                    "project_id": "project_id",
                    "prompt_version": "prompt_version",
                },
            },
            tags=["string", "string", "string"],
        )
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_raw_response_replace(self, async_client: AsyncBraintrust) -> None:
        response = await async_client.functions.with_raw_response.replace(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        function = await response.parse()
        assert_matches_type(Function, function, path=["response"])

    @parametrize
    async def test_streaming_response_replace(self, async_client: AsyncBraintrust) -> None:
        async with async_client.functions.with_streaming_response.replace(
            function_data={"type": "prompt"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            slug="slug",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            function = await response.parse()
            assert_matches_type(Function, function, path=["response"])

        assert cast(Any, response.is_closed) is True
